<?php

namespace App\Console\Commands;

use App\Models\Report\DailyReport;
use App\Models\Report\MonthlyReport;
use App\Models\Report\WeeklyReport;
use App\Models\UserLog;
use App\Repositories\MailerRepository;
use Illuminate\Console\Command;

class SendWarningCommand extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'send:email {type?}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Gửi mail cảnh báo';

    protected $mailer;
    protected $receiver;
    protected $admin;

    /**
     * Create a new command instance.
     *
     * SendWarningCommand constructor.
     * @param MailerRepository $mailerRepository
     */
    public function __construct(MailerRepository $mailerRepository)
    {
        parent::__construct();
        $this->mailer = $mailerRepository;
        $this->receiver = 'tuvt.ttdt@gmail.com';
        $this->admin = 'tuvuongcms';
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        $type = $this->argument('type');

        if (empty($type) || !in_array($type, ['daily_checking', 'warning_daily_report', 'warning_weekly_report', 'warning_monthly_report', 'reporting'])) {
            $type = $this->choice(
                'Chọn loại cảnh báo ',
                [
                    'daily_checking',
                    'warning_daily_report',
                    'warning_weekly_report',
                    'warning_monthly_report',
                    'reporting'
                ]
            );
        }

        echo "\n Bắt đầu xử lý gửi mail cảnh báo.";

        switch ($type) {
            case 'daily_checking':
                $check = UserLog::where('account_input', $this->admin)
                    ->where('time_login', '>=', date('Y-m-d 06:00:00', time()))
                    ->where('time_login', '<=', date('Y-m-d 09:00:00', time()))
                    ->count();
                if ($check < 1) {
                    $this->mailer->sendMail(
                        $this->receiver,
                        'Cảnh báo: Bạn chưa vào check CMS đầu giờ hàng ngày trước 9h sáng.',
                        'Cảnh báo: Bạn chưa vào check CMS đầu giờ hàng ngày trước 9h sáng.'
                    );
                    echo "\n - Vừa gửi mail cảnh báo không check hàng ngày.";
                }
                break;
            case 'warning_daily_report':
                $check = DailyReport::where('date', date('Y-m-d', time() - 86400))->count();
                if ($check < 100) {
                    $this->mailer->sendMail(
                        $this->receiver,
                        'Cảnh báo lỗi: BXH Ngày chưa được tính toán.',
                        'Cảnh báo lỗi: BXH Ngày chưa được tính toán.'
                    );
                    echo "\n - Vừa gửi mail cảnh báo BXH Ngày xong.";
                }
                break;
            case 'warning_weekly_report':
                if (strtoupper(date('D')) == 'SUN') {
                    $check = WeeklyReport::where('week', date('W'))
                        ->where('year', date('Y'))
                        ->count();
                    if ($check < 100) {
                        $this->mailer->sendMail(
                            $this->receiver,
                            'Cảnh báo lỗi: BXH Tuần chưa được tính toán.',
                            'Cảnh báo lỗi: BXH Tuần chưa được tính toán.'
                        );
                        echo "\n - Vừa gửi mail cảnh báo BXH Tuần xong.";
                    }
                }
                break;
            case 'warning_monthly_report':
                if (date('d') == 21) {
                    $check = MonthlyReport::where('month', date('m'))
                        ->where('year', date('Y'))
                        ->count();
                    if ($check < 100) {
                        $this->mailer->sendMail(
                            $this->receiver,
                            'Cảnh báo lỗi: BXH Tháng chưa được tính toán.',
                            'Cảnh báo lỗi: BXH Tháng chưa được tính toán.'
                        );
                        echo "\n - Vừa gửi mail cảnh báo BXH Tuần xong.";
                    }
                }
                break;
            case 'reporting':

                break;

            default:
        }

        echo "\n Kết thúc xử lý gửi mail cảnh báo. \n";

    }
}
