<?php

namespace App\Http\Controllers\Api;

use App\Repositories\ListenRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;

class ListenHistoryController extends Controller
{
    protected $listenRepository;

    public function __construct(ListenRepository $listenRepository)
    {
        ini_set('max_execution_time', -1);
        $this->listenRepository = $listenRepository;
    }

    public function getListenHistoryByDateBk(Request $request)
    {
        $rules = [
            'date' => [
                'required',
                function($attibute, $value, $fail){die($value);
                    $array_date = explode('-', $value);
                    if (is_array($array_date) && count($array_date) == 3) {
                        if (! checkdate(trim($array_date[1]), trim($array_date[2]), trim($array_date[0]))) {
                            return $fail('Ngày không hợp lệ.');
                        }
                    } else {
                        return $fail('Ngày phải có định dạng yyyy-mm-dd.');
                    }
                }
            ]
        ];

        $messages = [
            'date.required' => 'Truyền thiếu tham số date (yyyy-mm-dd)'
        ];

        $validator = Validator::make($request->all(), $rules, $messages);

        if ($validator->fails() ){
            return [
                'code' => 400,
                'message' => $validator->errors()->first(),
                'data' => null
            ];
        }

        $date = request('date');

        return response()->json(dataResponse(
            200,
            'Lấy thông tin thành công',
            $this->listenRepository->getListenHistoryByDate($date)
        ));

    }

    /**
     * API lấy lịch sử lượt nghe theo từng ngày
     * http://localhost:8000/api/get-listen-history-by-date?date=2020-03-30
     * http://crawler.vmusicchart.vn/api/get-listen-history-by-date?date=2020-03-30
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function getListenHistoryByDate(Request $request)
    {
        $date = request('date');

        $array_date = explode('-', $date);
        if (is_array($array_date) && count($array_date) == 3) {
            if (! checkdate(trim($array_date[1]), trim($array_date[2]), trim($array_date[0]))) {
                return response()->json(dataResponse(
                    200,
                    'Lấy thông tin thành công',
                    null
                ));
            }
        } else {
            return response()->json(dataResponse(
                200,
                'Lấy thông tin thành công',
                null
            ));
        }

        return response()->json(dataResponse(
            200,
            'Lấy thông tin thành công',
            $this->listenRepository->getListenHistoryByDate($date)
        ));
    }
}
