<?php

namespace App\Http\Controllers\Api;

use App\Models\Track;
use App\Repositories\TrackRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Maatwebsite\Excel\Facades\Excel;

class NctController extends Controller
{
    protected $trackRepository;

    public function __construct(TrackRepository $trackRepository)
    {
        $this->trackRepository = $trackRepository;
        ini_set('max_execution_time', -1);
    }

    /**
     * API: http://localhost:8000/api/get-nct-top100
     * Deploy: https://vmusicchart.vn/crawler/api/get-nct-top100
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function getNctTop100(Request $request)
    {
        $url = config('api.nct.top100');
        $url_playlist_top100 = config('api.nct.playlist_top100');

        $tracks = $this->trackRepository->processNctGetTrackInfoFromTop100($url);
        $tracks_playlist = $this->trackRepository->processNctGetTrackInfoFromTop100Playlist($url_playlist_top100);
        $tracks_nct = Track::select(['id', 'id_nct', 'composer_list'])
            ->whereNotNull('id_nct')
            ->where('id_nct', '<>', '')
            ->get();
        $array_tracks_nct = [];
        foreach ($tracks_nct as $track) {
            $array_tracks_nct[$track['id_nct']] = $track['composer_list'];
        }

        $response = null;
        foreach ($tracks as $key => $track) {
            $link = null;
            $duration = 0;
            $check = $this->privateGeTrackLink($tracks_playlist, $track['id_nct']);
            if ($check) {
                $link = $check['link'];
                $duration = $check['duration'];
            }
            $response[] = $track;
            $response[$key]['link'] = $link;
            $response[$key]['duration'] = $duration;
            $response[$key]['str_duration'] = gmdate('i:s', $duration);
            if (array_key_exists($track['id_nct'], $array_tracks_nct)) {
                $response[$key]['composer'] = $array_tracks_nct[$track['id_nct']];
            } else {
                $response[$key]['composer'] = '?';
            }
        }

        return response()->json([
            'code' => 200,
            'message' => 'Lấy thông tin top 100 bài hát NCT thành công.',
            'data' => $response
        ]);
    }

    public function getNctTop100AsComma(Request $request)
    {
        $url = config('api.nct.top100');
        $url_playlist_top100 = config('api.nct.playlist_top100');

        $tracks = $this->trackRepository->processNctGetTrackInfoFromTop100($url);
        $tracks_playlist = $this->trackRepository->processNctGetTrackInfoFromTop100Playlist($url_playlist_top100);
        $tracks_nct = Track::select(['id', 'id_nct', 'composer_list'])
            ->whereNotNull('id_nct')
            ->where('id_nct', '<>', '')
            ->get();
        $array_tracks_nct = [];
        foreach ($tracks_nct as $track) {
            $array_tracks_nct[$track['id_nct']] = $track['composer_list'];
        }

        $response = null;

        Excel::create('weekly_report', function($excel) use ($tracks){
            $excel->sheet('Sheetname', function($sheet) use ($tracks){
                // Sheet manipulation
                foreach ($tracks as $key => $val) {
                    $sheet->cell('A'.($key+1), function($cell) use ($val){
                        $cell->setValue($val['title']);
                    });
                    $sheet->cell('B'.($key+1), function($cell) use ($val){
                        $cell->setValue($val['id_nct']);
                    });
                    $sheet->cell('C'.($key+1), function($cell) use ($val){
                        $cell->setValue($val['id_nct_str']);
                    });
                    $sheet->cell('D'.($key+1), function($cell) use ($val){
                        $cell->setValue($val['singer']);
                    });
                    $sheet->cell('E'.($key+1), function($cell) use ($val){
                        $cell->setValue($val['link']);
                    });
                }

                /*$sheet->cells('A1:A5', function($cells) {
                    // manipulate the range of cells
                });*/

            });
        })->store('xlsx', storage_path('exports/weekly_report'));


        /*foreach ($tracks as $key => $track) {
            $link = null;
            $duration = 0;
            $check = $this->privateGeTrackLink($tracks_playlist, $track['id_nct']);
            if ($check) {
                $link = $check['link'];
                $duration = $check['duration'];
            }
            $response[] = $track;
            $response[$key]['link'] = $link;
            $response[$key]['duration'] = $duration;
            $response[$key]['str_duration'] = gmdate('i:s', $duration);
            if (array_key_exists($track['id_nct'], $array_tracks_nct)) {
                $response[$key]['composer'] = $array_tracks_nct[$track['id_nct']];
            } else {
                $response[$key]['composer'] = '?';
            }

            echo  $track['title'] . "\t" . $track['id_nct'] . "\t" . $track['id_nct_str'] . "\t" . $track['singer'] . '<br>';
        }*/

        die();

        /*return response()->json([
            'code' => 200,
            'message' => 'Lấy thông tin top 100 bài hát NCT thành công.',
            'data' => $response
        ]);*/
    }

    /**
     * API: http://localhost:8000/api/get-zing-top100
     * Deploy: https://vmusicchart.vn/crawler/api/get-zing-top100
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function getZingTop100(Request $request)
    {
        $url = config('api.zing.playlist');
        $curl = curlZingMp3($url);
        $response = json_decode($curl);
        //
        $array_tracks = [];
        if (isset($response->data) && isset($response->data->song) && isset($response->data->song->items)) {
            $tracks =  $response->data->song->items;
            if ($tracks) {
                $index = 0;
                foreach ($tracks as $key => $track) {
                    if ($key >= 50) {
                        $index = 1;
                    }
                    $array_tracks[$index][] = $track;
                    //$zing_id = $track->id;
                    //$api_url = getZingSongInfoUrl($zing_id);
                    //$array_composer[] = getZingSongInfo($api_url);
                }
            }
        }
        //
        $array_composer = [];
        if (count($array_tracks) > 0) {
            foreach ($array_tracks as $index => $tracks) {
                foreach ($tracks as $key => $track) {
                    $zing_id = $track->id;
                    $api_url = getZingSongInfoUrl($zing_id);
                    $array_composer[] = getZingSongInfo($api_url);
                }
            }
        }

        return response()->json([
            'code' => 200,
            'message' => 'Lấy thông tin top 100 bài hát Zing thành công.',
            'data' => $array_composer
        ]);
    }

    private function privateGeTrackLink($tracks_playlist, $track_id)
    {
        if (is_array($tracks_playlist) && count($tracks_playlist) > 0) {
            foreach ($tracks_playlist as $key => $val) {
                if ($val['id_nct'] == $track_id) {
                    return $val;
                }
            }
        }

        return null;
    }

}
